@@ -3,7 +3,6 @@ use crate::{DependentJoint, Distribution, IndependentJoint, RandomVariable};
 use rand::prelude::*;
 use rand_distr::Dirichlet as RandDirichlet;
 use rayon::{iter::IntoParallelIterator, prelude::*};
-use special::Gamma;
 use std::{ops::BitAnd, ops::Mul};
 
 /// Dirichlet distribution
@@ -22,19 +21,11 @@ pub enum DirichletError {
     Unknown,
 }
 
-fn multivariate_beta(alpha: &[f64]) -> f64 {
-    alpha
-        .into_par_iter()
-        .map(|&alphai| Gamma::gamma(alphai))
-        .product::<f64>()
-        / Gamma::gamma(alpha.into_par_iter().sum::<f64>())
-}
-
 impl Distribution for Dirichlet {
-    type T = Vec<f64>;
-    type U = DirichletParams;
+    type Value = Vec<f64>;
+    type Condition = DirichletParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let alpha = theta.alpha();
 
         if x.len() != alpha.len() {
@@ -43,14 +34,17 @@ impl Distribution for Dirichlet {
             ));
         }
 
-        Ok(1.0 / multivariate_beta(alpha)
-            * x.into_par_iter()
-                .zip(alpha.into_par_iter())
-                .map(|(&xi, &alphai)| xi.powf(alphai - 1.0))
-                .product::<f64>())
+        Ok(x.into_par_iter()
+            .zip(alpha.into_par_iter())
+            .map(|(&xi, &alphai)| xi.powf(alphai - 1.0))
+            .product::<f64>())
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let alpha = theta.alpha();
 
         let dirichlet = match RandDirichlet::new(alpha) {
@@ -90,9 +84,31 @@ impl DirichletParams {
     }
 }
 
+impl RandomVariable for DirichletParams {
+    type RestoreInfo = usize;
+
+    fn transform_vec(&self) -> (Vec<f64>, Self::RestoreInfo) {
+        let n = self.alpha.len();
+        (self.clone().alpha, n)
+    }
+
+    fn len(&self) -> usize {
+        self.alpha.len()
+    }
+
+    fn restore(v: &[f64], info: &Self::RestoreInfo) -> Result<Self, DistributionError> {
+        let n = *info;
+        if n < 2 {
+            return Err(DistributionError::InvalidRestoreVector);
+        }
+        let alpha = v.to_vec();
+        Self::new(alpha)
+    }
+}
+
 impl<Rhs, TRhs> Mul<Rhs> for Dirichlet
 where
-    Rhs: Distribution<T = TRhs, U = DirichletParams>,
+    Rhs: Distribution<Value = TRhs, Condition = DirichletParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<Self, Rhs, Vec<f64>, TRhs, DirichletParams>;
@@ -104,7 +120,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for Dirichlet
 where
-    Rhs: Distribution<T = DirichletParams, U = URhs>,
+    Rhs: Distribution<Value = DirichletParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output = DependentJoint<Self, Rhs, Vec<f64>, DirichletParams, URhs>;
@@ -116,8 +132,19 @@ where
 
 #[cfg(test)]
 mod tests {
+    use crate::{Dirichlet, DirichletParams, Distribution};
+    use rand::prelude::*;
     #[test]
     fn it_works() {
-        assert_eq!(2 + 2, 4);
+        let dirichlet = Dirichlet;
+        let mut rng = StdRng::from_seed([1; 32]);
+
+        let alpha = vec![1.0, 1.0, 2.0, 3.0, 4.0, 5.0];
+
+        let x = dirichlet
+            .sample(&DirichletParams::new(alpha).unwrap(), &mut rng)
+            .unwrap();
+
+        println!("{:#?}", x);
     }
 }
