@@ -1,8 +1,10 @@
-use crate::DistributionError;
+use crate::{
+    ConditionDifferentiableDistribution, DistributionError, ValueDifferentiableDistribution,
+};
 use crate::{DependentJoint, Distribution, IndependentJoint, RandomVariable};
 use rand::prelude::*;
 use rand_distr::Beta as RandBeta;
-use special::Beta as BetaFunc;
+use special::Gamma;
 use std::{ops::BitAnd, ops::Mul};
 
 /// Beta distribution
@@ -18,18 +20,21 @@ pub enum BetaError {
 }
 
 impl Distribution for Beta {
-    type T = f64;
-    type U = BetaParams;
+    type Value = f64;
+    type Condition = BetaParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let alpha = theta.alpha();
         let beta = theta.beta();
 
-        Ok((x.powf(alpha - 1.0) * (1.0 - x).powf(beta - 1.0))
-            / BetaFunc::ln_beta(alpha, beta).exp())
+        Ok(x.powf(alpha - 1.0) * (1.0 - x).powf(beta - 1.0))
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let alpha = theta.alpha();
         let beta = theta.beta();
 
@@ -42,6 +47,33 @@ impl Distribution for Beta {
     }
 }
 
+impl ValueDifferentiableDistribution for Beta {
+    fn ln_diff_value(
+        &self,
+        x: &Self::Value,
+        theta: &Self::Condition,
+    ) -> Result<Vec<f64>, DistributionError> {
+        let alpha = theta.alpha();
+        let beta = theta.beta();
+        let f_x = (alpha - 1.0) / x - (beta - 1.0) / (1.0 - x);
+        Ok(vec![f_x])
+    }
+}
+
+impl ConditionDifferentiableDistribution for Beta {
+    fn ln_diff_condition(
+        &self,
+        x: &Self::Value,
+        theta: &Self::Condition,
+    ) -> Result<Vec<f64>, DistributionError> {
+        let alpha = theta.alpha();
+        let beta = theta.beta();
+        let f_alpha = x.ln() - alpha.digamma() + (alpha + beta).digamma();
+        let f_beta = (1.0 - x).ln() - beta.digamma() + (alpha + beta).digamma();
+        Ok(vec![f_alpha, f_beta])
+    }
+}
+
 #[derive(Clone, Debug, PartialEq)]
 pub struct BetaParams {
     alpha: f64,
@@ -73,9 +105,25 @@ impl BetaParams {
     }
 }
 
+impl RandomVariable for BetaParams {
+    type RestoreInfo = ();
+
+    fn transform_vec(&self) -> (Vec<f64>, Self::RestoreInfo) {
+        (vec![self.alpha, self.beta], ())
+    }
+
+    fn len(&self) -> usize {
+        2usize
+    }
+
+    fn restore(v: &[f64], _: &Self::RestoreInfo) -> Result<Self, DistributionError> {
+        Self::new(v[0], v[1])
+    }
+}
+
 impl<Rhs, TRhs> Mul<Rhs> for Beta
 where
-    Rhs: Distribution<T = TRhs, U = BetaParams>,
+    Rhs: Distribution<Value = TRhs, Condition = BetaParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<Self, Rhs, f64, TRhs, BetaParams>;
@@ -87,7 +135,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for Beta
 where
-    Rhs: Distribution<T = BetaParams, U = URhs>,
+    Rhs: Distribution<Value = BetaParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output = DependentJoint<Self, Rhs, f64, BetaParams, URhs>;
@@ -99,8 +147,21 @@ where
 
 #[cfg(test)]
 mod tests {
+    use crate::{Beta, BetaParams, Distribution};
+    use rand::prelude::*;
+
     #[test]
     fn it_works() {
-        assert_eq!(2 + 2, 4);
+        let n = Beta;
+        let mut rng = StdRng::from_seed([1; 32]);
+
+        let alpha = 2.0;
+        let beta = 3.0;
+
+        let x = n
+            .sample(&BetaParams::new(alpha, beta).unwrap(), &mut rng)
+            .unwrap();
+
+        println!("{}", x);
     }
 }
