@@ -1,26 +1,32 @@
-use crate::{DependentJoint, Distribution, IndependentJoint, RandomVariable};
+use crate::{
+    ConditionDifferentiableDistribution, DependentJoint, Distribution, IndependentJoint,
+    NormalParams, RandomVariable, ValueDifferentiableDistribution,
+};
 use crate::{DistributionError, NormalError};
 use rand::prelude::*;
 use rand_distr::Normal as RandNormal;
-use std::{f64::consts::PI, ops::BitAnd, ops::Mul};
+use std::{ops::BitAnd, ops::Mul};
 
 /// Normal distribution
 #[derive(Clone, Debug)]
 pub struct Normal;
 
 impl Distribution for Normal {
-    type T = f64;
-    type U = NormalParams;
+    type Value = f64;
+    type Condition = NormalParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let mu = theta.mu();
         let sigma = theta.sigma();
 
-        Ok(1.0 / (2.0 * PI * sigma.powi(2)).sqrt()
-            * (-(x - mu).powi(2) / (2.0 * sigma.powi(2))).exp())
+        Ok((-(x - mu).powi(2) / (2.0 * sigma.powi(2))).exp())
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let mu = theta.mu();
         let sigma = theta.sigma();
 
@@ -37,35 +43,9 @@ impl Distribution for Normal {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct NormalParams {
-    mu: f64,
-    sigma: f64,
-}
-
-impl NormalParams {
-    pub fn new(mu: f64, sigma: f64) -> Result<Self, DistributionError> {
-        if sigma <= 0.0 {
-            return Err(DistributionError::InvalidParameters(
-                NormalError::SigmaMustBePositive.into(),
-            ));
-        }
-
-        Ok(Self { mu, sigma })
-    }
-
-    pub fn mu(&self) -> f64 {
-        self.mu
-    }
-
-    pub fn sigma(&self) -> f64 {
-        self.sigma
-    }
-}
-
 impl<Rhs, TRhs> Mul<Rhs> for Normal
 where
-    Rhs: Distribution<T = TRhs, U = NormalParams>,
+    Rhs: Distribution<Value = TRhs, Condition = NormalParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<Self, Rhs, f64, TRhs, NormalParams>;
@@ -77,7 +57,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for Normal
 where
-    Rhs: Distribution<T = NormalParams, U = URhs>,
+    Rhs: Distribution<Value = NormalParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output = DependentJoint<Self, Rhs, f64, NormalParams, URhs>;
@@ -87,6 +67,33 @@ where
     }
 }
 
+impl ValueDifferentiableDistribution for Normal {
+    fn ln_diff_value(
+        &self,
+        x: &Self::Value,
+        theta: &Self::Condition,
+    ) -> Result<Vec<f64>, DistributionError> {
+        let sigma = theta.sigma();
+        let mu = theta.mu();
+        let f_x = -(x - mu) / sigma.powi(2);
+        Ok(vec![f_x])
+    }
+}
+
+impl ConditionDifferentiableDistribution for Normal {
+    fn ln_diff_condition(
+        &self,
+        x: &Self::Value,
+        theta: &Self::Condition,
+    ) -> Result<Vec<f64>, DistributionError> {
+        let sigma = theta.sigma();
+        let mu = theta.mu();
+        let f_mu = (x - mu) / sigma.powi(2);
+        let f_sigma = (x - mu).powi(2) / sigma.powi(3);
+        Ok(vec![f_mu, f_sigma])
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::{Distribution, Normal, NormalParams};
