@@ -4,6 +4,10 @@ use rand::prelude::*;
 use rand_distr::Exp as RandExp;
 use std::{ops::BitAnd, ops::Mul};
 
+pub mod params;
+
+pub use params::*;
+
 /// Exponential distribution
 #[derive(Clone, Debug)]
 pub struct Exp;
@@ -15,16 +19,20 @@ pub enum ExpError {
 }
 
 impl Distribution for Exp {
-    type T = f64;
-    type U = ExpParams;
+    type Value = f64;
+    type Condition = ExpParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let lambda = theta.lambda();
 
         Ok(lambda * (-lambda * x).exp())
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let lambda = theta.lambda();
 
         let exp = match RandExp::new(lambda) {
@@ -36,30 +44,9 @@ impl Distribution for Exp {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct ExpParams {
-    lambda: f64,
-}
-
-impl ExpParams {
-    pub fn new(lambda: f64) -> Result<Self, DistributionError> {
-        if lambda <= 0.0 {
-            return Err(DistributionError::InvalidParameters(
-                ExpError::LambdaMustBePositive.into(),
-            ));
-        }
-
-        Ok(Self { lambda })
-    }
-
-    pub fn lambda(&self) -> f64 {
-        self.lambda
-    }
-}
-
 impl<Rhs, TRhs> Mul<Rhs> for Exp
 where
-    Rhs: Distribution<T = TRhs, U = ExpParams>,
+    Rhs: Distribution<Value = TRhs, Condition = ExpParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<Self, Rhs, f64, TRhs, ExpParams>;
@@ -71,7 +58,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for Exp
 where
-    Rhs: Distribution<T = ExpParams, U = URhs>,
+    Rhs: Distribution<Value = ExpParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output = DependentJoint<Self, Rhs, f64, ExpParams, URhs>;
