@@ -1,10 +1,9 @@
-use crate::DistributionError;
 use crate::{DependentJoint, Distribution, IndependentJoint, RandomVariable};
+use crate::{DistributionError, WishartParams};
 use crate::{ExactMultivariateNormalParams, MultivariateNormal};
+use opensrdk_linear_algebra::pp::trf::PPTRF;
 use opensrdk_linear_algebra::*;
 use rand::prelude::*;
-use special::Gamma;
-use std::f64::consts::PI;
 use std::{ops::BitAnd, ops::Mul};
 
 /// Wishart distribution
@@ -19,39 +18,31 @@ pub enum WishartError {
     NMustBeGTEDimension,
 }
 
-fn multivariate_gamma(p: u64, a: f64) -> f64 {
-    PI.powf(p as f64 * (p as f64 - 1.0) / 4.0)
-        * (0..p)
-            .into_iter()
-            .map(|i| Gamma::gamma(a + i as f64 / 2.0))
-            .product::<f64>()
-}
-
 impl Distribution for Wishart {
-    type T = Matrix;
-    type U = WishartParams;
+    type Value = PPTRF;
+    type Condition = WishartParams;
 
     /// x must be cholesky decomposed
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let lv = theta.lv();
         let n = theta.n();
 
-        let p = x.rows() as f64;
+        let p = x.0.dim() as f64;
+        let lx = x.0.to_mat();
 
-        Ok(
-            x.trdet().powf((n + p + 1.0) / 2.0) * (-0.5 * lv.clone().potrs(x * x.t())?.tr()).exp()
-                / (2f64.powf(n * p / 2.0)
-                    * lv.trdet().powf(n / 2.0)
-                    * multivariate_gamma(p as u64, n / 2.0)),
-        )
+        Ok(lx.trdet().powf(n + p + 1.0) * (-0.5 * lv.clone().pptrs(&lx * lx.t())?.tr()).exp())
     }
 
     /// output is cholesky decomposed
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let lv = theta.lv();
         let n = theta.n() as usize;
 
-        let p = lv.rows();
+        let p = lv.0.dim();
 
         let normal = MultivariateNormal::new();
         let normal_params = ExactMultivariateNormalParams::new(vec![0.0; p], lv.clone())?;
@@ -67,48 +58,19 @@ impl Distribution for Wishart {
                 },
             )?;
 
-        Ok(w)
-    }
-}
-
-#[derive(Clone, Debug, PartialEq)]
-pub struct WishartParams {
-    lv: Matrix,
-    n: f64,
-}
-
-impl WishartParams {
-    pub fn new(lv: Matrix, n: f64) -> Result<Self, DistributionError> {
-        let p = lv.rows();
-        if p != lv.cols() {
-            return Err(DistributionError::InvalidParameters(
-                WishartError::DimensionMismatch.into(),
-            ));
-        }
-        if n <= p as f64 - 1.0 as f64 {
-            return Err(DistributionError::InvalidParameters(
-                WishartError::NMustBeGTEDimension.into(),
-            ));
-        }
-
-        Ok(Self { lv, n })
-    }
-
-    pub fn lv(&self) -> &Matrix {
-        &self.lv
-    }
-
-    pub fn n(&self) -> f64 {
-        self.n
+        Ok(SymmetricPackedMatrix::from_mat(&w)
+            .unwrap()
+            .pptrf()
+            .unwrap())
     }
 }
 
 impl<Rhs, TRhs> Mul<Rhs> for Wishart
 where
-    Rhs: Distribution<T = TRhs, U = WishartParams>,
+    Rhs: Distribution<Value = TRhs, Condition = WishartParams>,
     TRhs: RandomVariable,
 {
-    type Output = IndependentJoint<Self, Rhs, Matrix, TRhs, WishartParams>;
+    type Output = IndependentJoint<Self, Rhs, PPTRF, TRhs, WishartParams>;
 
     fn mul(self, rhs: Rhs) -> Self::Output {
         IndependentJoint::new(self, rhs)
@@ -117,10 +79,10 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for Wishart
 where
-    Rhs: Distribution<T = WishartParams, U = URhs>,
+    Rhs: Distribution<Value = WishartParams, Condition = URhs>,
     URhs: RandomVariable,
 {
-    type Output = DependentJoint<Self, Rhs, Matrix, WishartParams, URhs>;
+    type Output = DependentJoint<Self, Rhs, PPTRF, WishartParams, URhs>;
 
     fn bitand(self, rhs: Rhs) -> Self::Output {
         DependentJoint::new(self, rhs)
