@@ -1,8 +1,7 @@
-use crate::DistributionError;
+use crate::{ChiSquaredParams, DistributionError};
 use crate::{DependentJoint, Distribution, IndependentJoint, RandomVariable};
 use rand::prelude::*;
 use rand_distr::ChiSquared as RandChiSquared;
-use special::Gamma;
 use std::{ops::BitAnd, ops::Mul};
 
 /// Chi squared distribution
@@ -16,17 +15,20 @@ pub enum ChiSquaredError {
 }
 
 impl Distribution for ChiSquared {
-    type T = f64;
-    type U = ChiSquaredParams;
+    type Value = f64;
+    type Condition = ChiSquaredParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let k = theta.k();
 
-        Ok((1.0 / (2f64.powf(k / 2.0) * Gamma::gamma(k / 2.0)))
-            * (x.powf(k / 2.0 - 1.0) * (-x / 2.0).exp()))
+        Ok(x.powf(k / 2.0 - 1.0) * (-x / 2.0).exp())
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let k = theta.k();
 
         let chi_squared = match RandChiSquared::new(k) {
@@ -38,30 +40,9 @@ impl Distribution for ChiSquared {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct ChiSquaredParams {
-    k: f64,
-}
-
-impl ChiSquaredParams {
-    pub fn new(k: f64) -> Result<Self, DistributionError> {
-        if k <= 0.0 {
-            return Err(DistributionError::InvalidParameters(
-                ChiSquaredError::KMustBePositive.into(),
-            ));
-        }
-
-        Ok(Self { k })
-    }
-
-    pub fn k(&self) -> f64 {
-        self.k
-    }
-}
-
 impl<Rhs, TRhs> Mul<Rhs> for ChiSquared
 where
-    Rhs: Distribution<T = TRhs, U = ChiSquaredParams>,
+    Rhs: Distribution<Value = TRhs, Condition = ChiSquaredParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<Self, Rhs, f64, TRhs, ChiSquaredParams>;
@@ -73,7 +54,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for ChiSquared
 where
-    Rhs: Distribution<T = ChiSquaredParams, U = URhs>,
+    Rhs: Distribution<Value = ChiSquaredParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output = DependentJoint<Self, Rhs, f64, ChiSquaredParams, URhs>;
