@@ -9,23 +9,24 @@ use crate::{opensrdk_linear_algebra::*, RandomVariable};
 use crate::{DistributionError, EllipticalParams};
 use ey::y_ey;
 use opensrdk_kernel_method::*;
+use opensrdk_linear_algebra::matrix::ge::sy_he::po::trf::POTRF;
 
 #[derive(Clone, Debug)]
 pub struct ExactEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     base: BaseEllipticalProcessParams<K, T>,
     mu: Vec<f64>,
-    lsigma: Matrix,
+    lsigma: POTRF,
     sigma_inv_y: Matrix,
     mahalanobis_squared: f64,
 }
 
 impl<K, T> ExactEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn new(base: BaseEllipticalProcessParams<K, T>, y: &[f64]) -> Result<Self, DistributionError> {
@@ -49,7 +50,7 @@ where
         let y_ey = y_ey(y, ey).col_mat();
         let y_ey_t = y_ey.t();
         let sigma_inv_y = lsigma.potrs(y_ey)?;
-        let mahalanobis_squared = (y_ey_t * &sigma_inv_y)[0][0];
+        let mahalanobis_squared = (y_ey_t * &sigma_inv_y)[(0, 0)];
 
         Ok(Self {
             base,
@@ -63,7 +64,7 @@ where
 
 impl<K, T> BaseEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     /// Elliptical Process without approximation for scalability.
@@ -76,9 +77,29 @@ where
     }
 }
 
+impl<K, T> RandomVariable for ExactEllipticalProcessParams<K, T>
+where
+    K: PositiveDefiniteKernel<T>,
+    T: RandomVariable,
+{
+    type RestoreInfo = ();
+
+    fn transform_vec(&self) -> (Vec<f64>, Self::RestoreInfo) {
+        todo!()
+    }
+
+    fn len(&self) -> usize {
+        todo!()
+    }
+
+    fn restore(v: &[f64], info: &Self::RestoreInfo) -> Result<Self, DistributionError> {
+        todo!()
+    }
+}
+
 impl<K, T> EllipticalParams for ExactEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn mu(&self) -> &Vec<f64> {
@@ -89,22 +110,18 @@ where
         Ok(self.lsigma.potrs(v)?)
     }
 
-    fn sigma_det_sqrt(&self) -> f64 {
-        self.lsigma.trdet()
-    }
-
     fn lsigma_cols(&self) -> usize {
-        self.lsigma.cols()
+        self.lsigma.0.cols()
     }
 
     fn sample(&self, z: Vec<f64>) -> Result<Vec<f64>, DistributionError> {
-        Ok((self.mu[0] + &self.lsigma * z.col_mat()).vec())
+        Ok((self.mu[0] + &self.lsigma.0 * z.col_mat()).vec())
     }
 }
 
 impl<K, T> EllipticalProcessParams<K, T> for ExactEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn mahalanobis_squared(&self) -> f64 {
